"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import abc
import collections.abc
from nidaqmx._stubs import data_moniker_pb2
import grpc
import grpc.aio
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore[misc, type-arg]
    ...

class DataMonikerStub:
    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    BeginSidebandStream: grpc.UnaryUnaryMultiCallable[
        data_moniker_pb2.BeginMonikerSidebandStreamRequest,
        data_moniker_pb2.BeginMonikerSidebandStreamResponse,
    ]

    StreamReadWrite: grpc.StreamStreamMultiCallable[
        data_moniker_pb2.MonikerWriteRequest,
        data_moniker_pb2.MonikerReadResponse,
    ]

    StreamRead: grpc.UnaryStreamMultiCallable[
        data_moniker_pb2.MonikerList,
        data_moniker_pb2.MonikerReadResponse,
    ]

    StreamWrite: grpc.StreamStreamMultiCallable[
        data_moniker_pb2.MonikerWriteRequest,
        data_moniker_pb2.StreamWriteResponse,
    ]

class DataMonikerAsyncStub:
    BeginSidebandStream: grpc.aio.UnaryUnaryMultiCallable[
        data_moniker_pb2.BeginMonikerSidebandStreamRequest,
        data_moniker_pb2.BeginMonikerSidebandStreamResponse,
    ]

    StreamReadWrite: grpc.aio.StreamStreamMultiCallable[
        data_moniker_pb2.MonikerWriteRequest,
        data_moniker_pb2.MonikerReadResponse,
    ]

    StreamRead: grpc.aio.UnaryStreamMultiCallable[
        data_moniker_pb2.MonikerList,
        data_moniker_pb2.MonikerReadResponse,
    ]

    StreamWrite: grpc.aio.StreamStreamMultiCallable[
        data_moniker_pb2.MonikerWriteRequest,
        data_moniker_pb2.StreamWriteResponse,
    ]

class DataMonikerServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def BeginSidebandStream(
        self,
        request: data_moniker_pb2.BeginMonikerSidebandStreamRequest,
        context: _ServicerContext,
    ) -> typing.Union[data_moniker_pb2.BeginMonikerSidebandStreamResponse, collections.abc.Awaitable[data_moniker_pb2.BeginMonikerSidebandStreamResponse]]: ...

    @abc.abstractmethod
    def StreamReadWrite(
        self,
        request_iterator: _MaybeAsyncIterator[data_moniker_pb2.MonikerWriteRequest],
        context: _ServicerContext,
    ) -> typing.Union[collections.abc.Iterator[data_moniker_pb2.MonikerReadResponse], collections.abc.AsyncIterator[data_moniker_pb2.MonikerReadResponse]]: ...

    @abc.abstractmethod
    def StreamRead(
        self,
        request: data_moniker_pb2.MonikerList,
        context: _ServicerContext,
    ) -> typing.Union[collections.abc.Iterator[data_moniker_pb2.MonikerReadResponse], collections.abc.AsyncIterator[data_moniker_pb2.MonikerReadResponse]]: ...

    @abc.abstractmethod
    def StreamWrite(
        self,
        request_iterator: _MaybeAsyncIterator[data_moniker_pb2.MonikerWriteRequest],
        context: _ServicerContext,
    ) -> typing.Union[collections.abc.Iterator[data_moniker_pb2.StreamWriteResponse], collections.abc.AsyncIterator[data_moniker_pb2.StreamWriteResponse]]: ...

def add_DataMonikerServicer_to_server(servicer: DataMonikerServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nidaqmx._stubs import data_moniker_pb2 as data__moniker__pb2


class DataMonikerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BeginSidebandStream = channel.unary_unary(
                '/ni.data_monikers.DataMoniker/BeginSidebandStream',
                request_serializer=data__moniker__pb2.BeginMonikerSidebandStreamRequest.SerializeToString,
                response_deserializer=data__moniker__pb2.BeginMonikerSidebandStreamResponse.FromString,
                )
        self.StreamReadWrite = channel.stream_stream(
                '/ni.data_monikers.DataMoniker/StreamReadWrite',
                request_serializer=data__moniker__pb2.MonikerWriteRequest.SerializeToString,
                response_deserializer=data__moniker__pb2.MonikerReadResponse.FromString,
                )
        self.StreamRead = channel.unary_stream(
                '/ni.data_monikers.DataMoniker/StreamRead',
                request_serializer=data__moniker__pb2.MonikerList.SerializeToString,
                response_deserializer=data__moniker__pb2.MonikerReadResponse.FromString,
                )
        self.StreamWrite = channel.stream_stream(
                '/ni.data_monikers.DataMoniker/StreamWrite',
                request_serializer=data__moniker__pb2.MonikerWriteRequest.SerializeToString,
                response_deserializer=data__moniker__pb2.StreamWriteResponse.FromString,
                )


class DataMonikerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BeginSidebandStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamReadWrite(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamWrite(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DataMonikerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BeginSidebandStream': grpc.unary_unary_rpc_method_handler(
                    servicer.BeginSidebandStream,
                    request_deserializer=data__moniker__pb2.BeginMonikerSidebandStreamRequest.FromString,
                    response_serializer=data__moniker__pb2.BeginMonikerSidebandStreamResponse.SerializeToString,
            ),
            'StreamReadWrite': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamReadWrite,
                    request_deserializer=data__moniker__pb2.MonikerWriteRequest.FromString,
                    response_serializer=data__moniker__pb2.MonikerReadResponse.SerializeToString,
            ),
            'StreamRead': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamRead,
                    request_deserializer=data__moniker__pb2.MonikerList.FromString,
                    response_serializer=data__moniker__pb2.MonikerReadResponse.SerializeToString,
            ),
            'StreamWrite': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamWrite,
                    request_deserializer=data__moniker__pb2.MonikerWriteRequest.FromString,
                    response_serializer=data__moniker__pb2.StreamWriteResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ni.data_monikers.DataMoniker', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DataMoniker(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BeginSidebandStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ni.data_monikers.DataMoniker/BeginSidebandStream',
            data__moniker__pb2.BeginMonikerSidebandStreamRequest.SerializeToString,
            data__moniker__pb2.BeginMonikerSidebandStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamReadWrite(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/ni.data_monikers.DataMoniker/StreamReadWrite',
            data__moniker__pb2.MonikerWriteRequest.SerializeToString,
            data__moniker__pb2.MonikerReadResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamRead(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/ni.data_monikers.DataMoniker/StreamRead',
            data__moniker__pb2.MonikerList.SerializeToString,
            data__moniker__pb2.MonikerReadResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamWrite(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/ni.data_monikers.DataMoniker/StreamWrite',
            data__moniker__pb2.MonikerWriteRequest.SerializeToString,
            data__moniker__pb2.StreamWriteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from __future__ import annotations

import contextlib
import errno
import importlib.resources as pkg_resources
import json
import logging
import os
import pathlib
import re
import shutil
import subprocess  # nosec: B404
import sys
import tempfile
import traceback
import requests
import zipfile
from typing import Generator, List, Optional, Tuple
from urllib.parse import urlparse

import click

if sys.platform.startswith("win"):
    import winreg
elif sys.platform.startswith("linux"):
    import distro

    from nidaqmx._linux_installation_commands import (
        LINUX_COMMANDS,
        get_linux_installation_commands,
    )

_logger = logging.getLogger(__name__)

METADATA_FILE = "_installer_metadata.json"
_NETWORK_TIMEOUT_IN_SECONDS = 60


def _parse_version(version: str) -> tuple[int, ...]:
    """
    Split the version string into a tuple of integers.

    >>> _parse_version("23.8.0")
    (23, 8, 0)
    >>> _parse_version("24.0.0")
    (24, 0, 0)
    >>> _parse_version("invalid_version")
    Traceback (most recent call last):
    ...
    click.exceptions.ClickException: Invalid version format found
    """
    try:
        return tuple(int(part) for part in version.split("."))
    except ValueError as e:
        _logger.info("Failed to parse version.", exc_info=True)
        raise click.ClickException(f"Invalid version number '{version}'.") from e


def _get_daqmx_installed_version() -> str | None:
    """
    Check for existing installation of NI-DAQmx.

    """
    if sys.platform.startswith("win"):
        try:
            _logger.debug("Reading the registry entries to get installed DAQmx version")
            with winreg.OpenKeyEx(
                winreg.HKEY_LOCAL_MACHINE,
                r"SOFTWARE\National Instruments\NI-DAQmx\CurrentVersion",
                0,
                winreg.KEY_READ | winreg.KEY_WOW64_32KEY,
            ) as daqmx_reg_key:
                product_name = winreg.QueryValueEx(daqmx_reg_key, "ProductName")[0]
                product_version = winreg.QueryValueEx(daqmx_reg_key, "Version")[0]

            if product_name == "NI-DAQmx":
                _logger.info(
                    "Found registry entries for Product Name: %s and version %s",
                    product_name,
                    product_version,
                )
                return product_version
            return None
        except FileNotFoundError:
            _logger.info("No existing NI-DAQmx installation found.")
            return None
        except PermissionError as e:
            _logger.info("Failed to read the registry key.", exc_info=True)
            raise click.ClickException(
                f"Permission denied while getting the installed NI-DAQmx version.\nDetails: {e}"
            ) from e
        except OSError as e:
            _logger.info("Failed to read the registry key.", exc_info=True)
            raise click.ClickException(
                f"An OS error occurred while getting the installed NI-DAQmx version.\nDetails: {e}"
            ) from e
    elif sys.platform.startswith("linux"):
        try:
            distribution = distro.id()
            _logger.debug("Checking for installed NI-DAQmx version")
            commands_info = LINUX_COMMANDS[distribution]
            query_command = commands_info.get_daqmx_version
            # Run the package query command defined by _linux_installation_commands.py.
            query_output = subprocess.run(query_command, stdout=subprocess.PIPE, text=True).stdout  # nosec: B603

            if distribution == "ubuntu":
                version_match = re.search(r"ii\s+ni-daqmx\s+(\d+\.\d+\.\d+)", query_output)
            elif distribution == "opensuse" or distribution == "rhel":
                version_match = re.search(r"ni-daqmx-(\d+\.\d+\.\d+)", query_output)
            else:
                raise click.ClickException(f"Unsupported distribution '{distribution}'")
            if version_match is None:
                return None
            else:
                installed_version = version_match.group(1)
                _logger.info("Found installed NI-DAQmx version: %s", installed_version)
                return installed_version

        except subprocess.CalledProcessError as e:
            _logger.info("Failed to get installed NI-DAQmx version.", exc_info=True)
            raise click.ClickException(
                f"An error occurred while getting the installed NI-DAQmx version.\nCommand returned non-zero exit status '{e.returncode}'."
            ) from e
    else:
        raise NotImplementedError("This function is only supported on Windows and Linux.")


# Creating a temp file that we then close and yield - this is used to allow subprocesses to access
@contextlib.contextmanager
def _multi_access_temp_file(
    *, suffix: str = ".exe", delete: bool = True
) -> Generator[str]:
    """
    Context manager for creating a temporary file.

    """
    try:
        temp_file = tempfile.NamedTemporaryFile(suffix=suffix, delete=False, mode="w")
        temp_file.close()
        _logger.debug("Created temp file: %s", temp_file.name)
    except Exception as e:
        _logger.info("Failed to create temporary file.", exc_info=True)
        raise click.ClickException(
            f"Failed to create temporary file '{temp_file.name}'.\nDetails: {e}"
        ) from e

    try:
        yield temp_file.name
    finally:
        if delete:
            try:
                _logger.debug("Deleting temp file: %s", temp_file.name)
                os.unlink(temp_file.name)
            except ValueError as e:
                _logger.info("Failed to delete temporary file.", exc_info=True)
                raise click.ClickException(
                    f"Failed to delete temporary file '{temp_file.name}'.\nDetails: {e}"
                ) from e


def _load_data(
    json_data: str, platform: str
) -> tuple[str | None, str | None, str | None, list[str] | None]:
    """
    Load data from JSON string and extract Windows metadata.

    >>> json_data = '{"Windows": [{"Location": "path/to/windows/driver", "Version": "24.0", "Release": "2024Q1", "supportedOS": ["Windows 11"]}], "Linux": []}'
    >>> _load_data(json_data, "Windows")
    ('path/to/windows/driver', '24.0', '2024Q1', ['Windows 11'])

    >>> json_data = '{"Windows": [], "Linux": [{"Location": "path/to/linux/driver", "Version": "24.0", "Release": "2024Q1", "supportedOS": ["ubuntu 20.04 ", "rhel 9"]}]}'
    >>> _load_data(json_data, "Linux")
    ('path/to/linux/driver', '24.0', '2024Q1', ['ubuntu 20.04', 'rhel 9'])

    >>> json_data = '{"Windows": [{"Location": "path/to/windows/driver", "Version": "24.0", "Release": "2024Q1", "supportedOS": ["Windows 11"]}], "Linux": []}'
    >>> _load_data(json_data, "Linux")
    Traceback (most recent call last):
    click.exceptions.ClickException: Unable to fetch driver details

    >>> json_data = 'invalid json'
    >>> _load_data(json_data, "Windows")
    Traceback (most recent call last):
    click.exceptions.ClickException: Failed to parse the driver metadata.
    Details: Expecting value: line 1 column 1 (char 0)

    >>> json_data = '{"Windows": [{"Location": "path/to/windows/driver", "Version": "24.0", "Release": "2024Q1", "supportedOS": ["Windows 11"]}], "Linux": []}'
    >>> _load_data(json_data, "macOS")
    Traceback (most recent call last):
    click.exceptions.ClickException: Unsupported os 'macOS'

    """
    try:
        if platform == "Windows":
            metadata = json.loads(json_data).get("Windows", [])
        elif platform == "Linux":
            metadata = json.loads(json_data).get("Linux", [])
        else:
            raise click.ClickException(f"Unsupported os '{platform}'")
    except json.JSONDecodeError as e:
        _logger.info("Failed to parse the json data.", exc_info=True)
        raise click.ClickException(f"Failed to parse the driver metadata.\nDetails: {e}") from e

    for metadata_entry in metadata:
        location: str | None = metadata_entry.get("Location")
        version: str | None = metadata_entry.get("Version")
        release: str | None = metadata_entry.get("Release")
        supported_os: list[str] | None = metadata_entry.get("supportedOS")
        _logger.debug("From metadata file found location %s and version %s.", location, version)
        if location and version:
            return location, version, release, supported_os
    raise click.ClickException(f"Unable to fetch driver details")


def _get_driver_details(
    platform: str,
) -> tuple[str | None, str | None, str | None, list[str] | None]:
    """
    Parse the JSON data and retrieve the download link and version information.

    """
    try:
        with pkg_resources.open_text(__package__, METADATA_FILE) as json_file:
            _logger.debug("Opening the metadata file %s.", METADATA_FILE)
            location, version, release, supported_os = _load_data(json_file.read(), platform)
        return location, version, release, supported_os

    except click.ClickException:
        raise
    except Exception as e:
        _logger.info("Failed to get driver metadata.", exc_info=True)
        raise click.ClickException(
            f"An error occurred while getting the driver metadata.\nDetails: {e}"
        ) from e


def _install_daqmx_driver_windows_core(download_url: str) -> None:
    """
    Download and launch NI-DAQmx Driver installation in an interactive mode

    """
    _validate_download_url(download_url)
    try:
        with _multi_access_temp_file() as temp_file:
            _logger.info("Downloading Driver to %s", temp_file)
            response = requests.get(download_url, timeout=_NETWORK_TIMEOUT_IN_SECONDS)
            response.raise_for_status()
            with open(temp_file, 'wb') as f:
                f.write(response.content)
            _logger.info("Installing NI-DAQmx")
            # Run the installer that we just downloaded from https://download.ni.com.
            subprocess.run([temp_file], shell=True, check=True)  # nosec: B602
    except subprocess.CalledProcessError as e:
        _logger.info("Failed to install NI-DAQmx driver.", exc_info=True)
        raise click.ClickException(
            f"An error occurred while installing the NI-DAQmx driver. Command returned non-zero exit status '{e.returncode}'."
        ) from e
    except requests.RequestException as e:
        _logger.info("Failed to download NI-DAQmx driver.", exc_info=True)
        raise click.ClickException(f"Failed to download the NI-DAQmx driver.\nDetails: {e}") from e
    except Exception as e:
        _logger.info("Failed to install NI-DAQmx driver.", exc_info=True)
        raise click.ClickException(f"Failed to install the NI-DAQmx driver.\nDetails: {e}") from e


def _install_daqmx_driver_linux_core(download_url: str, release: str) -> None:
    """
    Download NI Linux Device Drivers and install NI-DAQmx on Linux OS

    """
    if sys.platform.startswith("linux"):
        _validate_download_url(download_url)
        try:
            with _multi_access_temp_file(suffix=".zip") as temp_file:
                _logger.info("Downloading Driver to %s", temp_file)
                response = requests.get(download_url, timeout=_NETWORK_TIMEOUT_IN_SECONDS)
                response.raise_for_status()
                with open(temp_file, 'wb') as f:
                    f.write(response.content)

                with tempfile.TemporaryDirectory() as temp_folder:
                    directory_to_extract_to = temp_folder

                    _logger.info("Unzipping the downloaded file to %s", directory_to_extract_to)

                    with zipfile.ZipFile(temp_file, "r") as zip_ref:
                        zip_ref.extractall(directory_to_extract_to)

                    _logger.info("Installing NI-DAQmx")
                    for command in get_linux_installation_commands(
                        directory_to_extract_to, distro.id(), distro.version(), release
                    ):
                        print("\nRunning command:", " ".join(command))
                        # Run the commands defined in
                        # _linux_installation_commands.py to install the package
                        # that we just downloaded from https://download.ni.com.
                        subprocess.run(command, check=True)  # nosec: B603

            # Check if the installation was successful
            if not _get_daqmx_installed_version():
                raise click.ClickException(
                    "Failed to install NI-DAQmx driver. All installation commands ran successfully but the driver is not installed."
                )
            else:
                print("NI-DAQmx driver installed successfully. Please reboot the system.")

        except subprocess.CalledProcessError as e:
            _logger.info("Failed to install NI-DAQmx driver.", exc_info=True)
            raise click.ClickException(
                f"An error occurred while installing the NI-DAQmx driver. Command returned non-zero exit status '{e.returncode}'."
            ) from e
        except requests.RequestException as e:
            _logger.info("Failed to download NI-DAQmx driver.", exc_info=True)
            raise click.ClickException(
                f"Failed to download the NI-DAQmx driver.\nDetails: {e}"
            ) from e
        except Exception as e:
            _logger.info("Failed to install NI-DAQmx driver.", exc_info=True)
            raise click.ClickException(
                f"Failed to install the NI-DAQmx driver.\nDetails: {e}"
            ) from e
    else:
        raise NotImplementedError("This function is only supported on Linux.")


def _validate_download_url(download_url: str) -> None:
    """Velidate that the download URL uses https and points to a trusted site."""
    parsed_url = urlparse(download_url)
    if parsed_url.scheme != "https" or parsed_url.netloc != "download.ni.com":
        raise click.ClickException(f"Unsupported download URL: {download_url}")


def _ask_user_confirmation(user_message: str) -> bool:
    """
    Prompt for user confirmation

    """
    while True:
        response = input(user_message + " (yes/no): ").strip().lower()
        if response in ["yes", "y"]:
            return True
        elif response in ["no", "n"]:
            return False
        else:
            print("Please enter 'yes' or 'no'.")


def _upgrade_daqmx_user_confirmation(
    installed_version: str,
    latest_version: str,
    release: str,
) -> bool:
    """
    Confirm with the user and return the user response.

    """
    _logger.debug("Entering _upgrade_daqmx_user_confirmation")
    installed_parts = _parse_version(installed_version)
    latest_parts = _parse_version(latest_version)
    if installed_parts >= latest_parts:
        print(
            f"Installed NI-DAQmx version ({installed_version}) is up to date. (Expected {latest_version} ({release}) or newer.)"
        )
        return False
    is_upgrade = _ask_user_confirmation(
        f"Installed NI-DAQmx version is {installed_version}. Latest version available is {latest_version} ({release}). Do you want to upgrade?"
    )
    return is_upgrade


def _fresh_install_daqmx_user_confirmation(
    latest_version: str,
    release: str,
) -> bool:
    """
    Confirm with the user and return the user response.

    """
    _logger.debug("Entering _fresh_install_daqmx_user_confirmation")
    return _ask_user_confirmation(
        f"Latest NI-DAQmx version available is {latest_version} ({release}). Do you want to install?"
    )


def _is_distribution_supported() -> None:
    """
    Raises an exception if the linux distribution and its version are not supported.

    """
    if sys.platform.startswith("linux"):
        dist_name = distro.id()
        dist_version = distro.version()

        # For rhel, we only need the major version
        if dist_name == "rhel":
            dist_version = dist_version.split(".")[0]
        dist_name_and_version = dist_name + " " + dist_version

        download_url, latest_version, release, supported_os = _get_driver_details("Linux")
        if supported_os is None:
            raise ValueError("supported_os cannot be None")

        # Check if the platform is one of the supported ones
        if dist_name_and_version in supported_os:
            _logger.info(f"The platform is supported: {dist_name_and_version}")
        else:
            raise click.ClickException(f"The platform {dist_name_and_version} is not supported.")
    else:
        raise NotImplementedError("This function is only supported on Linux.")


def _install_daqmx_driver():
    """
    Install the NI-DAQmx driver.

    """
    if sys.platform.startswith("win"):
        _logger.info("Windows platform detected")
        platform = "Windows"
    elif sys.platform.startswith("linux"):
        _logger.info("Linux platform detected")
        platform = "Linux"

        try:
            _is_distribution_supported()
        except Exception as e:
            raise click.ClickException(f"Distribution not supported.\nDetails: {e}") from e

    else:
        raise click.ClickException(
            f"The 'installdriver' command is supported only on Windows and Linux."
        )

    installed_version = _get_daqmx_installed_version()
    download_url, latest_version, release, _ = _get_driver_details(platform)

    if not download_url:
        raise click.ClickException(f"Failed to fetch the download url.")
    if not release or not latest_version:
        raise click.ClickException(f"Failed to fetch the release version string.")
    else:
        if installed_version:
            user_response = _upgrade_daqmx_user_confirmation(
                installed_version, latest_version, release
            )
        else:
            user_response = _fresh_install_daqmx_user_confirmation(
                latest_version, release
            )

        if user_response:
            if platform == "Linux":
                _install_daqmx_driver_linux_core(download_url, release)
            else:
                _install_daqmx_driver_windows_core(download_url)


def installdriver() -> None:
    """
    Download and launch NI-DAQmx Driver installation in an interactive mode.

    """
    try:
        _install_daqmx_driver()
    except click.ClickException:
        raise
    except Exception as e:
        _logger.info("Failed to install driver.", exc_info=True)
        raise click.ClickException(
            f"An error occurred during the installation of the NI-DAQmx driver.\nDetails: {e}"
        ) from e



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _SidebandStrategy:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SidebandStrategyEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SidebandStrategy.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _SidebandStrategy.ValueType  # 0
    GRPC: _SidebandStrategy.ValueType  # 1
    SHARED_MEMORY: _SidebandStrategy.ValueType  # 2
    DOUBLE_BUFFERED_SHARED_MEMORY: _SidebandStrategy.ValueType  # 3
    SOCKETS: _SidebandStrategy.ValueType  # 4
    SOCKETS_LOW_LATENCY: _SidebandStrategy.ValueType  # 5
    HYPERVISOR_SOCKETS: _SidebandStrategy.ValueType  # 6
    RDMA: _SidebandStrategy.ValueType  # 7
    RDMA_LOW_LATENCY: _SidebandStrategy.ValueType  # 8

class SidebandStrategy(_SidebandStrategy, metaclass=_SidebandStrategyEnumTypeWrapper): ...

UNKNOWN: SidebandStrategy.ValueType  # 0
GRPC: SidebandStrategy.ValueType  # 1
SHARED_MEMORY: SidebandStrategy.ValueType  # 2
DOUBLE_BUFFERED_SHARED_MEMORY: SidebandStrategy.ValueType  # 3
SOCKETS: SidebandStrategy.ValueType  # 4
SOCKETS_LOW_LATENCY: SidebandStrategy.ValueType  # 5
HYPERVISOR_SOCKETS: SidebandStrategy.ValueType  # 6
RDMA: SidebandStrategy.ValueType  # 7
RDMA_LOW_LATENCY: SidebandStrategy.ValueType  # 8
global___SidebandStrategy = SidebandStrategy

@typing.final
class BeginMonikerSidebandStreamRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STRATEGY_FIELD_NUMBER: builtins.int
    MONIKERS_FIELD_NUMBER: builtins.int
    strategy: global___SidebandStrategy.ValueType
    @property
    def monikers(self) -> global___MonikerList: ...
    def __init__(
        self,
        *,
        strategy: global___SidebandStrategy.ValueType = ...,
        monikers: global___MonikerList | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["monikers", b"monikers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["monikers", b"monikers", "strategy", b"strategy"]) -> None: ...

global___BeginMonikerSidebandStreamRequest = BeginMonikerSidebandStreamRequest

@typing.final
class BeginMonikerSidebandStreamResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STRATEGY_FIELD_NUMBER: builtins.int
    CONNECTION_URL_FIELD_NUMBER: builtins.int
    SIDEBAND_IDENTIFIER_FIELD_NUMBER: builtins.int
    BUFFER_SIZE_FIELD_NUMBER: builtins.int
    strategy: global___SidebandStrategy.ValueType
    connection_url: builtins.str
    sideband_identifier: builtins.str
    buffer_size: builtins.int
    def __init__(
        self,
        *,
        strategy: global___SidebandStrategy.ValueType = ...,
        connection_url: builtins.str = ...,
        sideband_identifier: builtins.str = ...,
        buffer_size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["buffer_size", b"buffer_size", "connection_url", b"connection_url", "sideband_identifier", b"sideband_identifier", "strategy", b"strategy"]) -> None: ...

global___BeginMonikerSidebandStreamResponse = BeginMonikerSidebandStreamResponse

@typing.final
class Moniker(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVICE_LOCATION_FIELD_NUMBER: builtins.int
    DATA_SOURCE_FIELD_NUMBER: builtins.int
    DATA_INSTANCE_FIELD_NUMBER: builtins.int
    service_location: builtins.str
    data_source: builtins.str
    data_instance: builtins.int
    def __init__(
        self,
        *,
        service_location: builtins.str = ...,
        data_source: builtins.str = ...,
        data_instance: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data_instance", b"data_instance", "data_source", b"data_source", "service_location", b"service_location"]) -> None: ...

global___Moniker = Moniker

@typing.final
class MonikerWriteRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MONIKERS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    @property
    def monikers(self) -> global___MonikerList: ...
    @property
    def data(self) -> global___MonikerValues: ...
    def __init__(
        self,
        *,
        monikers: global___MonikerList | None = ...,
        data: global___MonikerValues | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "monikers", b"monikers", "write_data", b"write_data"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "monikers", b"monikers", "write_data", b"write_data"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["write_data", b"write_data"]) -> typing.Literal["monikers", "data"] | None: ...

global___MonikerWriteRequest = MonikerWriteRequest

@typing.final
class MonikerReadResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    @property
    def data(self) -> global___MonikerValues: ...
    def __init__(
        self,
        *,
        data: global___MonikerValues | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___MonikerReadResponse = MonikerReadResponse

@typing.final
class MonikerList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    READ_MONIKERS_FIELD_NUMBER: builtins.int
    WRITE_MONIKERS_FIELD_NUMBER: builtins.int
    @property
    def read_monikers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Moniker]: ...
    @property
    def write_monikers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Moniker]: ...
    def __init__(
        self,
        *,
        read_monikers: collections.abc.Iterable[global___Moniker] | None = ...,
        write_monikers: collections.abc.Iterable[global___Moniker] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["read_monikers", b"read_monikers", "write_monikers", b"write_monikers"]) -> None: ...

global___MonikerList = MonikerList

@typing.final
class MonikerValues(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___MonikerValues = MonikerValues

@typing.final
class SidebandWriteRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CANCEL_FIELD_NUMBER: builtins.int
    VALUES_FIELD_NUMBER: builtins.int
    cancel: builtins.bool
    @property
    def values(self) -> global___MonikerValues: ...
    def __init__(
        self,
        *,
        cancel: builtins.bool = ...,
        values: global___MonikerValues | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["values", b"values"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["cancel", b"cancel", "values", b"values"]) -> None: ...

global___SidebandWriteRequest = SidebandWriteRequest

@typing.final
class SidebandReadResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CANCEL_FIELD_NUMBER: builtins.int
    VALUES_FIELD_NUMBER: builtins.int
    cancel: builtins.bool
    @property
    def values(self) -> global___MonikerValues: ...
    def __init__(
        self,
        *,
        cancel: builtins.bool = ...,
        values: global___MonikerValues | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["values", b"values"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["cancel", b"cancel", "values", b"values"]) -> None: ...

global___SidebandReadResponse = SidebandReadResponse

@typing.final
class StreamWriteResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StreamWriteResponse = StreamWriteResponse



# Do not edit this file; it was automatically generated.

import ctypes
import numpy
import pathlib

from nidaqmx import utils
from nidaqmx._bitfield_utils import enum_bitfield_to_list
from nidaqmx.utils import unflatten_channel_string
from nidaqmx.constants import (
    AOPowerUpOutputBehavior, AcquisitionType, LogicFamily, SensorPowerType,
    TerminalConfiguration, UsageTypeAI, UsageTypeAO, UsageTypeCI, UsageTypeCO,
    WriteBasicTEDSOptions, _TermCfg)

from typing import Optional, Union

__all__ = ['PhysicalChannel']


class PhysicalChannel:
    """
    Represents a DAQmx physical channel.
    """
    __slots__ = ['_name', '_interpreter', '__weakref__']

    def __init__(self, name, *, grpc_options=None):
        """
        Args:
            name (str): Specifies the name of the physical channel.
            grpc_options (Optional[:class:`~nidaqmx.GrpcSessionOptions`]): Specifies
                the gRPC session options.
        """
        self._name = name
        self._interpreter = utils._select_interpreter(grpc_options)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self._name == other._name
        return False

    def __hash__(self):
        return hash(self._name)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return f'PhysicalChannel(name={self._name})'

    @property
    def name(self):
        """
        str: Specifies the name of this physical channel.
        """
        return self._name

    @property
    def ai_input_srcs(self):
        """
        List[str]: Indicates the list of input sources supported by the
            channel. Channels may support using the signal from the I/O
            connector or one of several calibration signals.
        """

        val = self._interpreter.get_physical_chan_attribute_string(self._name, 0x2fd8)
        return unflatten_channel_string(val)

    @property
    def ai_meas_types(self):
        """
        List[:class:`nidaqmx.constants.UsageTypeAI`]: Indicates the
            measurement types supported by the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fd7)
        return [UsageTypeAI(e) for e in val]

    @property
    def ai_power_control_enable(self):
        """
        bool: Specifies whether to turn on the sensor's power supply.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x316d)
        return val

    @ai_power_control_enable.setter
    def ai_power_control_enable(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxSetPhysicalChanAIPowerControlEnable
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, c_bool32]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @ai_power_control_enable.deleter
    def ai_power_control_enable(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxResetPhysicalChanAIPowerControlEnable
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def ai_power_control_type(self):
        """
        :class:`nidaqmx.constants.SensorPowerType`: Specifies the type
            of power supplied to the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_int32(self._name, 0x316e)
        return SensorPowerType(val)

    @ai_power_control_type.setter
    def ai_power_control_type(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        val = val.value
        cfunc = lib_importer.windll.DAQmxSetPhysicalChanAIPowerControlType
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, ctypes.c_int]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @ai_power_control_type.deleter
    def ai_power_control_type(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxResetPhysicalChanAIPowerControlType
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def ai_power_control_voltage(self):
        """
        float: Specifies the voltage level for the sensor's power
            supply.
        """

        val = self._interpreter.get_physical_chan_attribute_double(self._name, 0x316c)
        return val

    @ai_power_control_voltage.setter
    def ai_power_control_voltage(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = (lib_importer.windll.
                 DAQmxSetPhysicalChanAIPowerControlVoltage)
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, ctypes.c_double]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @ai_power_control_voltage.deleter
    def ai_power_control_voltage(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = (lib_importer.windll.
                 DAQmxResetPhysicalChanAIPowerControlVoltage)
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def ai_sensor_power_open_chan(self):
        """
        bool: Indicates whether there is an open channel or undercurrent
            condition on the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x317c)
        return val

    @property
    def ai_sensor_power_overcurrent(self):
        """
        bool: Indicates whether there is an overcurrent condition on the
            channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x317d)
        return val

    @property
    def ai_sensor_power_types(self):
        """
        List[:class:`nidaqmx.constants.SensorPowerType`]: Indicates the
            types of power supplied to the sensor supported by this
            channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x3179)
        return [SensorPowerType(e) for e in val]

    @property
    def ai_sensor_power_voltage_range_vals(self):
        """
        List[float]: Indicates pairs of sensor power voltage ranges
            supported by this channel. Each pair consists of the low
            value followed by the high value.
        """

        val = self._interpreter.get_physical_chan_attribute_double_array(self._name, 0x317a)
        return val

    @property
    def ai_term_cfgs(self):
        """
        List[:class:`nidaqmx.constants.TerminalConfiguration`]:
            Indicates the list of terminal configurations supported by
            the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32(self._name, 0x2342)
        return enum_bitfield_to_list(
            val, _TermCfg, TerminalConfiguration)

    @property
    def ao_manual_control_amplitude(self):
        """
        float: Indicates the current value of the front panel amplitude
            control for the physical channel in volts.
        """

        val = self._interpreter.get_physical_chan_attribute_double(self._name, 0x2a1f)
        return val

    @property
    def ao_manual_control_enable(self):
        """
        bool: Specifies if you can control the physical channel
            externally via a manual control located on the device. You
            cannot simultaneously control a channel manually and with
            NI-DAQmx.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x2a1e)
        return val

    @ao_manual_control_enable.setter
    def ao_manual_control_enable(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = (lib_importer.windll.
                 DAQmxSetPhysicalChanAOManualControlEnable)
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, c_bool32]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @ao_manual_control_enable.deleter
    def ao_manual_control_enable(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = (lib_importer.windll.
                 DAQmxResetPhysicalChanAOManualControlEnable)
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def ao_manual_control_freq(self):
        """
        float: Indicates the current value of the front panel frequency
            control for the physical channel in hertz.
        """

        val = self._interpreter.get_physical_chan_attribute_double(self._name, 0x2a20)
        return val

    @property
    def ao_manual_control_short_detected(self):
        """
        bool: Indicates whether the physical channel is currently
            disabled due to a short detected on the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x2ec3)
        return val

    @property
    def ao_output_types(self):
        """
        List[:class:`nidaqmx.constants.UsageTypeAO`]: Indicates the
            output types supported by the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fd9)
        return [UsageTypeAO(e) for e in val]

    @property
    def ao_power_amp_channel_enable(self):
        """
        bool: Specifies whether to enable or disable a channel for
            amplification. This property can also be used to check if a
            channel is enabled.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x3062)
        return val

    @ao_power_amp_channel_enable.setter
    def ao_power_amp_channel_enable(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxSetAOPowerAmpChannelEnable
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, c_bool32]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @ao_power_amp_channel_enable.deleter
    def ao_power_amp_channel_enable(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxResetAOPowerAmpChannelEnable
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def ao_power_amp_gain(self):
        """
        float: Indicates the calibrated gain of the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_double(self._name, 0x3065)
        return val

    @property
    def ao_power_amp_offset(self):
        """
        float: Indicates the calibrated offset of the channel in volts.
        """

        val = self._interpreter.get_physical_chan_attribute_double(self._name, 0x3066)
        return val

    @property
    def ao_power_amp_overcurrent(self):
        """
        bool: Indicates if the channel detected an overcurrent
            condition.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x3064)
        return val

    @property
    def ao_power_amp_scaling_coeff(self):
        """
        List[float]: Indicates the coefficients of a polynomial equation
            used to scale from pre-amplified values.
        """

        val = self._interpreter.get_physical_chan_attribute_double_array(self._name, 0x3063)
        return val

    @property
    def ao_supported_power_up_output_types(self):
        """
        List[:class:`nidaqmx.constants.AOPowerUpOutputBehavior`]:
            Indicates the power up output types supported by the
            channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x304e)
        return [AOPowerUpOutputBehavior(e) for e in val]

    @property
    def ao_term_cfgs(self):
        """
        List[:class:`nidaqmx.constants.TerminalConfiguration`]:
            Indicates the list of terminal configurations supported by
            the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32(self._name, 0x29a3)
        return enum_bitfield_to_list(
            val, _TermCfg, TerminalConfiguration)

    @property
    def ci_meas_types(self):
        """
        List[:class:`nidaqmx.constants.UsageTypeCI`]: Indicates the
            measurement types supported by the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fda)
        return [UsageTypeCI(e) for e in val]

    @property
    def co_output_types(self):
        """
        List[:class:`nidaqmx.constants.UsageTypeCO`]: Indicates the
            output types supported by the channel.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fdb)
        return [UsageTypeCO(e) for e in val]

    @property
    def di_change_detect_supported(self):
        """
        bool: Indicates if the change detection timing type is supported
            for the digital input physical channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x29a6)
        return val

    @property
    def di_port_width(self):
        """
        int: Indicates in bits the width of digital input port.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x29a4)
        return val

    @property
    def di_samp_clk_supported(self):
        """
        bool: Indicates if the sample clock timing type is supported for
            the digital input physical channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x29a5)
        return val

    @property
    def di_samp_modes(self):
        """
        List[:class:`nidaqmx.constants.AcquisitionType`]: Indicates the
            sample modes supported by devices that support sample
            clocked digital input.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fe0)
        return [AcquisitionType(e) for e in val]

    @property
    def dig_port_logic_family(self):
        """
        :class:`nidaqmx.constants.LogicFamily`: Specifies the digital
            port logic family to use for acquisition and generation. A
            logic family corresponds to voltage thresholds that are
            compatible with a group of voltage standards. Refer to the
            device documentation for information on the logic high and
            logic low voltages for these logic families.
        """

        val = self._interpreter.get_physical_chan_attribute_int32(self._name, 0x31eb)
        return LogicFamily(val)

    @dig_port_logic_family.setter
    def dig_port_logic_family(self, val):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        val = val.value
        cfunc = lib_importer.windll.DAQmxSetPhysicalChanDigPortLogicFamily
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str, ctypes.c_int]
        error_code = cfunc(
            self._name, val)
        self._interpreter.check_for_error(error_code)

    @dig_port_logic_family.deleter
    def dig_port_logic_family(self):
        from nidaqmx._library_interpreter import LibraryInterpreter
        from nidaqmx._lib import lib_importer, ctypes_byte_str, c_bool32
        if not isinstance(self._interpreter, LibraryInterpreter):
            raise NotImplementedError
        cfunc = lib_importer.windll.DAQmxResetPhysicalChanDigPortLogicFamily
        if cfunc.argtypes is None:
            with cfunc.arglock:
                if cfunc.argtypes is None:
                    cfunc.argtypes = [
                        ctypes_byte_str]
        error_code = cfunc(
            self._name)
        self._interpreter.check_for_error(error_code)

    @property
    def do_port_width(self):
        """
        int: Indicates in bits the width of digital output port.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x29a7)
        return val

    @property
    def do_samp_clk_supported(self):
        """
        bool: Indicates if the sample clock timing type is supported for
            the digital output physical channel.
        """

        val = self._interpreter.get_physical_chan_attribute_bool(self._name, 0x29a8)
        return val

    @property
    def do_samp_modes(self):
        """
        List[:class:`nidaqmx.constants.AcquisitionType`]: Indicates the
            sample modes supported by devices that support sample
            clocked digital output.
        """

        val = self._interpreter.get_physical_chan_attribute_int32_array(self._name, 0x2fe1)
        return [AcquisitionType(e) for e in val]

    @property
    def teds_bit_stream(self):
        """
        List[int]: Indicates the TEDS binary bitstream without
            checksums.
        """

        val = self._interpreter.get_physical_chan_attribute_bytes(self._name, 0x21df)
        return val

    @property
    def teds_mfg_id(self):
        """
        int: Indicates the manufacturer ID of the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x21da)
        return val

    @property
    def teds_model_num(self):
        """
        int: Indicates the model number of the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x21db)
        return val

    @property
    def teds_serial_num(self):
        """
        int: Indicates the serial number of the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x21dc)
        return val

    @property
    def teds_template_ids(self):
        """
        List[int]: Indicates the IDs of the templates in the bitstream
            in **teds_bit_stream**.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32_array(self._name, 0x228f)
        return val

    @property
    def teds_version_letter(self):
        """
        str: Indicates the version letter of the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_string(self._name, 0x21de)
        return val

    @property
    def teds_version_num(self):
        """
        int: Indicates the version number of the sensor.
        """

        val = self._interpreter.get_physical_chan_attribute_uint32(self._name, 0x21dd)
        return val

    def clear_teds(self):
        """
        Removes TEDS information from the physical channel you specify.
        This function temporarily overrides any TEDS configuration for
        the physical channel that you performed in MAX.
        """

        self._interpreter.clear_teds(
            self._name)

    def configure_teds(
            self, file_path: Optional[Union[str, pathlib.PurePath]]=None):
        """
        Associates TEDS information with the physical channel you
        specify. If you do not specify the filename of a data sheet in
        the **file_path** input, this function attempts to find a TEDS
        sensor connected to the physical channel. This function
        temporarily overrides any TEDS configuration for the physical
        channel that you performed in MAX.

        Args:
            file_path: Is the path to a Virtual TEDS data sheet that
                you want to associate with the physical channel. If you
                do not specify anything for this input, this function
                attempts to find a TEDS sensor connected to the physical
                channel.
        """
        if file_path is None:
            file_path = ""


        self._interpreter.configure_teds(
            self._name, str(file_path))

    def write_to_teds_from_array(
            self, bit_stream=None,
            basic_teds_options=WriteBasicTEDSOptions.DO_NOT_WRITE):
        """
        Writes data from a 1D list of 8-bit unsigned integers to the
        TEDS sensor.

        Args:
            bit_stream (Optional[List[int]]): Is the TEDS bitstream to
                write to the sensor. This bitstream must be constructed
                according to the IEEE 1451.4 specification.
            basic_teds_options (Optional[nidaqmx.constants.WriteBasicTEDSOptions]): 
                Specifies how to handle basic TEDS data in the
                bitstream.
        """
        if bit_stream is None:
            bit_stream = []

        bit_stream = numpy.array(bit_stream, dtype=numpy.uint8)


        self._interpreter.write_to_teds_from_array(
            self._name, bit_stream, basic_teds_options.value)

    def write_to_teds_from_file(
            self, file_path: Optional[Union[str, pathlib.PurePath]]=None,
            basic_teds_options=WriteBasicTEDSOptions.DO_NOT_WRITE):
        """
        Writes data from a virtual TEDS file to the TEDS sensor.

        Args:
            file_path: Specifies the filename of a virtual TEDS file
                that contains the bitstream to write.
            basic_teds_options (Optional[nidaqmx.constants.WriteBasicTEDSOptions]): 
                Specifies how to handle basic TEDS data in the
                bitstream.
        """
        if file_path is None:
            file_path = ""


        self._interpreter.write_to_teds_from_file(
            self._name, str(file_path), basic_teds_options.value)



class _PhysicalChannelAlternateConstructor(PhysicalChannel):
    """
    Provide an alternate constructor for the PhysicalChannel object.

    This is a private API used to instantiate a PhysicalChannel with an existing interpreter.
    """
    # Setting __slots__ avoids TypeError: __class__ assignment: 'Base' object layout differs from 'Derived'.
    __slots__ = ()

    def __init__(self, name, interpreter):
        """
        Args:
            name: Specifies the name of the Physical Channel.
            interpreter: Specifies the interpreter instance.

        """
        self._name = name
        self._interpreter = interpreter

        # Use meta-programming to change the type of this object to PhysicalChannel,
        # so the user isn't confused when doing introspection.
        self.__class__ = PhysicalChannel  # type: ignore[assignment]


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_moniker.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x64\x61ta_moniker.proto\x12\x10ni.data_monikers\x1a\x19google/protobuf/any.proto\"\x8a\x01\n!BeginMonikerSidebandStreamRequest\x12\x34\n\x08strategy\x18\x01 \x01(\x0e\x32\".ni.data_monikers.SidebandStrategy\x12/\n\x08monikers\x18\x02 \x01(\x0b\x32\x1d.ni.data_monikers.MonikerList\"\xa4\x01\n\"BeginMonikerSidebandStreamResponse\x12\x34\n\x08strategy\x18\x01 \x01(\x0e\x32\".ni.data_monikers.SidebandStrategy\x12\x16\n\x0e\x63onnection_url\x18\x02 \x01(\t\x12\x1b\n\x13sideband_identifier\x18\x03 \x01(\t\x12\x13\n\x0b\x62uffer_size\x18\x04 \x01(\x12\"O\n\x07Moniker\x12\x18\n\x10service_location\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x61ta_source\x18\x02 \x01(\t\x12\x15\n\rdata_instance\x18\x03 \x01(\x03\"\x87\x01\n\x13MonikerWriteRequest\x12\x31\n\x08monikers\x18\x01 \x01(\x0b\x32\x1d.ni.data_monikers.MonikerListH\x00\x12/\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1f.ni.data_monikers.MonikerValuesH\x00\x42\x0c\n\nwrite_data\"D\n\x13MonikerReadResponse\x12-\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x1f.ni.data_monikers.MonikerValues\"r\n\x0bMonikerList\x12\x30\n\rread_monikers\x18\x02 \x03(\x0b\x32\x19.ni.data_monikers.Moniker\x12\x31\n\x0ewrite_monikers\x18\x03 \x03(\x0b\x32\x19.ni.data_monikers.Moniker\"5\n\rMonikerValues\x12$\n\x06values\x18\x01 \x03(\x0b\x32\x14.google.protobuf.Any\"W\n\x14SidebandWriteRequest\x12\x0e\n\x06\x63\x61ncel\x18\x01 \x01(\x08\x12/\n\x06values\x18\x02 \x01(\x0b\x32\x1f.ni.data_monikers.MonikerValues\"W\n\x14SidebandReadResponse\x12\x0e\n\x06\x63\x61ncel\x18\x01 \x01(\x08\x12/\n\x06values\x18\x02 \x01(\x0b\x32\x1f.ni.data_monikers.MonikerValues\"\x15\n\x13StreamWriteResponse*\xbd\x01\n\x10SidebandStrategy\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04GRPC\x10\x01\x12\x11\n\rSHARED_MEMORY\x10\x02\x12!\n\x1d\x44OUBLE_BUFFERED_SHARED_MEMORY\x10\x03\x12\x0b\n\x07SOCKETS\x10\x04\x12\x17\n\x13SOCKETS_LOW_LATENCY\x10\x05\x12\x16\n\x12HYPERVISOR_SOCKETS\x10\x06\x12\x08\n\x04RDMA\x10\x07\x12\x14\n\x10RDMA_LOW_LATENCY\x10\x08\x32\xb4\x03\n\x0b\x44\x61taMoniker\x12\x82\x01\n\x13\x42\x65ginSidebandStream\x12\x33.ni.data_monikers.BeginMonikerSidebandStreamRequest\x1a\x34.ni.data_monikers.BeginMonikerSidebandStreamResponse\"\x00\x12\x65\n\x0fStreamReadWrite\x12%.ni.data_monikers.MonikerWriteRequest\x1a%.ni.data_monikers.MonikerReadResponse\"\x00(\x01\x30\x01\x12V\n\nStreamRead\x12\x1d.ni.data_monikers.MonikerList\x1a%.ni.data_monikers.MonikerReadResponse\"\x00\x30\x01\x12\x61\n\x0bStreamWrite\x12%.ni.data_monikers.MonikerWriteRequest\x1a%.ni.data_monikers.StreamWriteResponse\"\x00(\x01\x30\x01\x42&\xf8\x01\x01\xaa\x02 NationalInstruments.DataMonikersb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'data_moniker_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001\252\002 NationalInstruments.DataMonikers'
  _SIDEBANDSTRATEGY._serialized_start=1037
  _SIDEBANDSTRATEGY._serialized_end=1226
  _BEGINMONIKERSIDEBANDSTREAMREQUEST._serialized_start=68
  _BEGINMONIKERSIDEBANDSTREAMREQUEST._serialized_end=206
  _BEGINMONIKERSIDEBANDSTREAMRESPONSE._serialized_start=209
  _BEGINMONIKERSIDEBANDSTREAMRESPONSE._serialized_end=373
  _MONIKER._serialized_start=375
  _MONIKER._serialized_end=454
  _MONIKERWRITEREQUEST._serialized_start=457
  _MONIKERWRITEREQUEST._serialized_end=592
  _MONIKERREADRESPONSE._serialized_start=594
  _MONIKERREADRESPONSE._serialized_end=662
  _MONIKERLIST._serialized_start=664
  _MONIKERLIST._serialized_end=778
  _MONIKERVALUES._serialized_start=780
  _MONIKERVALUES._serialized_end=833
  _SIDEBANDWRITEREQUEST._serialized_start=835
  _SIDEBANDWRITEREQUEST._serialized_end=922
  _SIDEBANDREADRESPONSE._serialized_start=924
  _SIDEBANDREADRESPONSE._serialized_end=1011
  _STREAMWRITERESPONSE._serialized_start=1013
  _STREAMWRITERESPONSE._serialized_end=1034
  _DATAMONIKER._serialized_start=1229
  _DATAMONIKER._serialized_end=1665
# @@protoc_insertion_point(module_scope)
